@@ -63,6 +63,7 @@
 #include "HttpHeader.h"
 #include "LogFactory.h"
 #include "CookieStorage.h"
+#include "AuthConfigFactory.h"
 
 namespace aria2 {
 
@@ -290,6 +291,8 @@ bool HttpResponseCommand::skipResponseBody
   if(httpResponse->getEntityLength() == 0 &&
      !httpResponse->isTransferEncodingSpecified()) {
     command->setStatusRealtime();
+    // If entity length == 0, then socket read/write check must be disabled.
+    command->disableSocketCheck();
     e->setNoWait(true);
   }
 
