@@ -33,6 +33,10 @@
  */
 /* copyright --> */
 #include "MultiDiskAdaptor.h"
+
+#include <algorithm>
+#include <cassert>
+
 #include "DefaultDiskWriter.h"
 #include "message.h"
 #include "Util.h"
@@ -44,8 +48,6 @@
 #include "StringFormat.h"
 #include "Logger.h"
 #include "SimpleRandomizer.h"
-#include <algorithm>
-#include <cassert>
 
 namespace aria2 {
 
@@ -208,14 +210,18 @@ void MultiDiskAdaptor::resetDiskWriterEntries()
 	((*itr)->getFileEntry()->getOffset()/pieceLength)*pieceLength;
       if(itr != diskWriterEntries.begin()) {
 	for(std::deque<SharedHandle<DiskWriterEntry> >::iterator i =
-	      itr-1; i != done; --i) {
+	      itr-1; true; --i) {
 	  const SharedHandle<FileEntry>& fileEntry = (*i)->getFileEntry();
-	  if((uint64_t)pieceStartOffset <
+	  if(pieceStartOffset <= fileEntry->getOffset() ||
+	     (uint64_t)pieceStartOffset <
 	     fileEntry->getOffset()+fileEntry->getLength()) {
 	    (*i)->needsFileAllocation(true);
 	  } else {
 	    break;
 	  }
+	  if(i == done) {
+	    break;
+	  }
 	}
       }
 
@@ -292,7 +298,12 @@ void MultiDiskAdaptor::openFile()
   _cachedTopDirPath = getTopDirPath();
   resetDiskWriterEntries();
   mkdir(_cachedTopDirPath);
-  // TODO we should call openIfNot here?
+  // Call DiskWriterEntry::openFile to make sure that zero-length files are
+  // created.
+  for(DiskWriterEntries::iterator itr = diskWriterEntries.begin();
+      itr != diskWriterEntries.end(); ++itr) {
+    openIfNot(*itr, &DiskWriterEntry::openFile, _cachedTopDirPath);
+  }
 }
 
 void MultiDiskAdaptor::initAndOpenFile()
@@ -372,6 +383,16 @@ findFirstDiskWriterEntry(const DiskWriterEntries& diskWriterEntries, off_t offse
   return first;
 }
 
+static void throwOnDiskWriterNotOpened(const SharedHandle<DiskWriterEntry>& e,
+				       off_t offset,
+				       const std::string& topDirPath)
+{
+  throw DlAbortEx
+    (StringFormat("DiskWriter for offset=%s, filename=%s is not opened.",
+		  Util::itos(offset).c_str(),
+		  e->getFilePath(topDirPath).c_str()).str());  
+}
+
 void MultiDiskAdaptor::writeData(const unsigned char* data, size_t len,
 				 off_t offset)
 {
@@ -384,6 +405,10 @@ void MultiDiskAdaptor::writeData(const unsigned char* data, size_t len,
 
     openIfNot(*i, &DiskWriterEntry::openFile, _cachedTopDirPath);
 
+    if(!(*i)->isOpen()) {
+      throwOnDiskWriterNotOpened(*i, offset+(len-rem), _cachedTopDirPath);
+    }
+
     (*i)->getDiskWriter()->writeData(data+(len-rem), writeLength, fileOffset);
     rem -= writeLength;
     fileOffset = 0;
@@ -405,6 +430,10 @@ ssize_t MultiDiskAdaptor::readData(unsigned char* data, size_t len, off_t offset
 
     openIfNot(*i, &DiskWriterEntry::openFile, _cachedTopDirPath);
 
+    if(!(*i)->isOpen()) {
+      throwOnDiskWriterNotOpened(*i, offset+(len-rem), _cachedTopDirPath);
+    }
+
     totalReadLength +=
       (*i)->getDiskWriter()->readData(data+(len-rem), readLength, fileOffset);
     rem -= readLength;
@@ -496,4 +525,10 @@ size_t MultiDiskAdaptor::utime(const Time& actime, const Time& modtime)
   return numOK;
 }
 
+const std::deque<SharedHandle<DiskWriterEntry> >&
+MultiDiskAdaptor::getDiskWriterEntries() const
+{
+  return diskWriterEntries;
+}
+
 } // namespace aria2
