@@ -146,7 +146,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_ENABLE_DIRECT_IO,
 				    TEXT_ENABLE_DIRECT_IO,
-				    V_FALSE));
+				    V_TRUE));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
@@ -429,6 +429,28 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
   // HTTP Specific Options
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_CA_CERTIFICATE,
+				    TEXT_CA_CERTIFICATE));
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_CERTIFICATE,
+				    TEXT_CERTIFICATE));
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_CHECK_CERTIFICATE,
+				    TEXT_CHECK_CERTIFICATE,
+				    V_FALSE));
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_ENABLE_HTTP_KEEP_ALIVE,
@@ -471,39 +493,6 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
-  {
-    SharedHandle<OptionHandler> op(new HttpProxyOptionHandler
-				   (PREF_HTTP_PROXY,
-				    TEXT_HTTP_PROXY,
-				    NO_DEFAULT_VALUE,
-				    PREF_HTTP_PROXY_HOST,
-				    PREF_HTTP_PROXY_PORT));
-    op->addTag(TAG_HTTP);
-    handlers.push_back(op);
-  }
-  {
-    SharedHandle<OptionHandler> op(new DefaultOptionHandler
-				   (PREF_HTTP_PROXY_USER,
-				    TEXT_HTTP_PROXY_USER));
-    op->addTag(TAG_HTTP);
-    handlers.push_back(op);
-  }
-  {
-    SharedHandle<OptionHandler> op(new DefaultOptionHandler
-				   (PREF_HTTP_PROXY_PASSWD,
-				    TEXT_HTTP_PROXY_PASSWD));
-    op->addTag(TAG_HTTP);
-    handlers.push_back(op);
-  }
-  {
-    SharedHandle<OptionHandler> op(new ParameterOptionHandler
-				   (PREF_HTTP_PROXY_METHOD,
-				    TEXT_HTTP_PROXY_METHOD,
-				    V_TUNNEL,
-				    V_GET, V_TUNNEL));
-    op->addTag(TAG_HTTP);
-    handlers.push_back(op);
-  }
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
 				   (PREF_HTTP_USER,
@@ -531,6 +520,13 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				    true));
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_PRIVATE_KEY,
+				    TEXT_PRIVATE_KEY));
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
 				   (PREF_REFERER,
@@ -559,7 +555,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_FTP_PASV,
 				    TEXT_FTP_PASV,
-				    V_FALSE)); // TODO ommit?
+				    V_TRUE));
     op->addTag(TAG_FTP);
     handlers.push_back(op);
   }
@@ -588,15 +584,6 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_FTP);
     handlers.push_back(op);
   }
-  {
-    SharedHandle<OptionHandler> op(new ParameterOptionHandler
-				   (PREF_FTP_VIA_HTTP_PROXY,
-				    TEXT_FTP_VIA_HTTP_PROXY,
-				    V_TUNNEL,
-				    V_GET, V_TUNNEL));
-    op->addTag(TAG_FTP);
-    handlers.push_back(op);
-  }
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
 				   (PREF_NETRC_PATH,
@@ -615,6 +602,58 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
+  // Proxy options
+  {
+    SharedHandle<OptionHandler> op(new HttpProxyOptionHandler
+				   (PREF_HTTP_PROXY,
+				    TEXT_HTTP_PROXY,
+				    NO_DEFAULT_VALUE));
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new HttpProxyOptionHandler
+				   (PREF_HTTPS_PROXY,
+				    TEXT_HTTPS_PROXY,
+				    NO_DEFAULT_VALUE));
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new HttpProxyOptionHandler
+				   (PREF_FTP_PROXY,
+				    TEXT_FTP_PROXY,
+				    NO_DEFAULT_VALUE));
+    op->addTag(TAG_FTP);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new HttpProxyOptionHandler
+				   (PREF_ALL_PROXY,
+				    TEXT_ALL_PROXY,
+				    NO_DEFAULT_VALUE));
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_NO_PROXY,
+				    TEXT_NO_PROXY));
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new ParameterOptionHandler
+				   (PREF_PROXY_METHOD,
+				    TEXT_PROXY_METHOD,
+				    V_TUNNEL,
+				    V_GET, V_TUNNEL));
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
   // BitTorrent/Metalink Options
   {
     SharedHandle<OptionHandler> op(new IntegerRangeOptionHandler
