@@ -33,6 +33,10 @@
  */
 /* copyright --> */
 #include "RequestGroup.h"
+
+#include <cassert>
+#include <algorithm>
+
 #include "PostDownloadHandler.h"
 #include "DownloadEngine.h"
 #include "DefaultSegmentManFactory.h"
@@ -41,7 +45,6 @@
 #include "Dependency.h"
 #include "prefs.h"
 #include "InitiateConnectionCommandFactory.h"
-#include "CUIDCounter.h"
 #include "File.h"
 #include "message.h"
 #include "Util.h"
@@ -87,7 +90,6 @@
 # include "BtPostDownloadHandler.h"
 # include "DHTSetup.h"
 # include "DHTRegistry.h"
-# include "PeerObject.h"
 # include "BtMessageFactory.h"
 # include "BtRequestFactory.h"
 # include "BtMessageDispatcher.h"
@@ -100,8 +102,6 @@
 #ifdef ENABLE_METALINK
 # include "MetalinkPostDownloadHandler.h"
 #endif // ENABLE_METALINK
-#include <cassert>
-#include <algorithm>
 
 namespace aria2 {
 
@@ -113,7 +113,7 @@ RequestGroup::RequestGroup(const Option* option,
 			   const std::deque<std::string>& uris):
   _gid(++_gidCounter),
   _uris(uris),
-  _numConcurrentCommand(0),
+  _numConcurrentCommand(option->getAsInt(PREF_SPLIT)),
   _numStreamConnection(0),
   _numCommand(0),
   _segmentManFactory(new DefaultSegmentManFactory(option)),
@@ -123,8 +123,9 @@ RequestGroup::RequestGroup(const Option* option,
   _forceHaltRequested(false),
   _singleHostMultiConnectionEnabled(true),
   _uriSelector(new InOrderURISelector()),
-  _lastModifiedTime(-1),
+  _lastModifiedTime(Time::null()),
   _fileNotFoundCount(0),
+  _inMemoryDownload(false),
   _option(option),
   _logger(LogFactory::getInstance())
 {
@@ -200,35 +201,46 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
 	_pieceStorage->setFileFilter(Util::parseIntRange(_option->get(PREF_SELECT_FILE)));
       }
       
-      BtProgressInfoFileHandle
+      SharedHandle<DefaultBtProgressInfoFile>
 	progressInfoFile(new DefaultBtProgressInfoFile(_downloadContext,
 						       _pieceStorage,
 						       _option));
       
-      BtRegistry::registerBtContext(btContext->getInfoHashAsString(), btContext);
-      BtRegistry::registerPieceStorage(btContext->getInfoHashAsString(),
+      SharedHandle<BtRegistry> btRegistry = e->getBtRegistry();
+
+      btRegistry->registerBtContext(btContext->getInfoHashAsString(),
+				    btContext);
+      btRegistry->registerPieceStorage(btContext->getInfoHashAsString(),
 				       _pieceStorage);
-      BtRegistry::registerBtProgressInfoFile(btContext->getInfoHashAsString(),
+      btRegistry->registerBtProgressInfoFile(btContext->getInfoHashAsString(),
 					     progressInfoFile);
 
   
       BtRuntimeHandle btRuntime(new BtRuntime());
       btRuntime->setListenPort(_option->getAsInt(PREF_LISTEN_PORT));
-      BtRegistry::registerBtRuntime(btContext->getInfoHashAsString(), btRuntime);
+      btRegistry->registerBtRuntime(btContext->getInfoHashAsString(),
+				    btRuntime);
+      _btRuntime = btRuntime;
+      progressInfoFile->setBtRuntime(btRuntime);
 
-      PeerStorageHandle peerStorage(new DefaultPeerStorage(btContext, _option));
-      BtRegistry::registerPeerStorage(btContext->getInfoHashAsString(), peerStorage);
+      SharedHandle<DefaultPeerStorage> peerStorage
+	(new DefaultPeerStorage(btContext, _option));
+      peerStorage->setBtRuntime(btRuntime);
+      peerStorage->setPieceStorage(_pieceStorage);
+      btRegistry->registerPeerStorage(btContext->getInfoHashAsString(),
+				      peerStorage);
+      _peerStorage = peerStorage;
+      progressInfoFile->setPeerStorage(peerStorage);
 
-      BtAnnounceHandle btAnnounce(new DefaultBtAnnounce(btContext, _option));
-      BtRegistry::registerBtAnnounce(btContext->getInfoHashAsString(), btAnnounce);
+      SharedHandle<DefaultBtAnnounce> btAnnounce
+	(new DefaultBtAnnounce(btContext, _option));
+      btAnnounce->setBtRuntime(btRuntime);
+      btAnnounce->setPieceStorage(_pieceStorage);
+      btAnnounce->setPeerStorage(peerStorage);
+      btRegistry->registerBtAnnounce(btContext->getInfoHashAsString(),
+				     btAnnounce);
       btAnnounce->shuffleAnnounce();
       
-      {
-	SharedHandle<PeerObjectCluster> po(new PeerObjectCluster());
-	BtRegistry::registerPeerObjectCluster(btContext->getInfoHashAsString(),
-					      po);
-      }
-
       // Remove the control file if download file doesn't exist
       if(progressInfoFile->exists() && !_pieceStorage->getDiskAdaptor()->fileExists()) {
 	progressInfoFile->removeFile();
@@ -269,7 +281,8 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
 	e->addCommand(commands);
 	if(!btContext->getNodes().empty() && DHTSetup::initialized()) {
 	  DHTEntryPointNameResolveCommand* command =
-	    new DHTEntryPointNameResolveCommand(CUIDCounterSingletonHolder::instance()->newID(), e, btContext->getNodes());
+	    new DHTEntryPointNameResolveCommand(e->newCUID(), e,
+						btContext->getNodes());
 	  command->setTaskQueue(DHTRegistry::_taskQueue);
 	  command->setTaskFactory(DHTRegistry::_taskFactory);
 	  command->setRoutingTable(DHTRegistry::_routingTable);
@@ -322,7 +335,7 @@ void RequestGroup::processCheckIntegrityEntry(std::deque<Command*>& commands,
     entry->initValidator();
     entry->cutTrailingGarbage();
     CheckIntegrityCommand* command =
-      new CheckIntegrityCommand(CUIDCounterSingletonHolder::instance()->newID(), this, e, entry);
+      new CheckIntegrityCommand(e->newCUID(), this, e, entry);
     commands.push_back(command);
   } else
 #endif // ENABLE_MESSAGE_DIGEST
@@ -489,6 +502,7 @@ void RequestGroup::createNextCommandWithAdj(std::deque<Command*>& commands,
     numCommand = 1+numAdj;
   } else {
     if(_numConcurrentCommand == 0) {
+      // TODO remove _uris.size() support
       numCommand = _uris.size();
     } else {
       numCommand = _numConcurrentCommand;
@@ -521,7 +535,9 @@ void RequestGroup::createNextCommand(std::deque<Command*>& commands,
 	req->setReferer(_option->get(PREF_REFERER));
 	req->setMethod(method);
 
-	Command* command = InitiateConnectionCommandFactory::createInitiateConnectionCommand(CUIDCounterSingletonHolder::instance()->newID(), req, this, e);
+	Command* command =
+	  InitiateConnectionCommandFactory::createInitiateConnectionCommand
+	  (e->newCUID(), req, this, e);
 	ServerHostHandle sv(new ServerHost(command->getCuid(), req->getHost()));
 	registerServerHost(sv);
 	// give a chance to be executed in the next loop in DownloadEngine
@@ -540,8 +556,9 @@ void RequestGroup::createNextCommand(std::deque<Command*>& commands,
 std::string RequestGroup::getFilePath() const
 {
   assert(!_downloadContext.isNull());
-  if(_downloadContext.isNull()) {
-    return A2STR::NIL;
+  if(inMemoryDownload()) {
+    static const std::string DIR_MEMORY("[MEMORY]");
+    return DIR_MEMORY+File(_downloadContext->getActualBasePath()).getBasename();
   } else {
     return _downloadContext->getActualBasePath();
   }
@@ -624,14 +641,8 @@ unsigned int RequestGroup::getNumConnection() const
 {
   unsigned int numConnection = _numStreamConnection;
 #ifdef ENABLE_BITTORRENT
-  {
-    BtContextHandle btContext = dynamic_pointer_cast<BtContext>(_downloadContext);
-    if(!btContext.isNull()) {
-      BtRuntimeHandle btRuntime = BT_RUNTIME(btContext);
-      if(!btRuntime.isNull()) {
-	numConnection += btRuntime->getConnections();
-      }
-    }
+  if(!_btRuntime.isNull()) {
+    numConnection += _btRuntime->getConnections();
   }
 #endif // ENABLE_BITTORRENT
   return numConnection;
@@ -652,14 +663,8 @@ TransferStat RequestGroup::calculateStat()
 {
   TransferStat stat;
 #ifdef ENABLE_BITTORRENT
-  {
-    BtContextHandle btContext = dynamic_pointer_cast<BtContext>(_downloadContext);
-    if(!btContext.isNull()) {
-      PeerStorageHandle peerStorage = PEER_STORAGE(btContext);
-      if(!peerStorage.isNull()) {
-	stat = peerStorage->calculateStat();
-      }
-    }
+  if(!_peerStorage.isNull()) {
+    stat = _peerStorage->calculateStat();
   }
 #endif // ENABLE_BITTORRENT
   if(!_segmentMan.isNull()) {
@@ -672,14 +677,8 @@ void RequestGroup::setHaltRequested(bool f)
 {
   _haltRequested = f;
 #ifdef ENABLE_BITTORRENT
-  {
-    BtContextHandle btContext = dynamic_pointer_cast<BtContext>(_downloadContext);
-    if(!btContext.isNull()) {
-      BtRuntimeHandle btRuntime = BT_RUNTIME(btContext);
-      if(!btRuntime.isNull()) {
-	btRuntime->setHalt(f);
-      }
-    }
+  if(!_btRuntime.isNull()) {
+    _btRuntime->setHalt(f);
   }
 #endif // ENABLE_BITTORRENT
 }
@@ -690,18 +689,21 @@ void RequestGroup::setForceHaltRequested(bool f)
   _forceHaltRequested = f;
 }
 
-void RequestGroup::releaseRuntimeResource()
+void RequestGroup::releaseRuntimeResource(DownloadEngine* e)
 {
 #ifdef ENABLE_BITTORRENT
   BtContextHandle btContext = dynamic_pointer_cast<BtContext>(_downloadContext);
   if(!btContext.isNull()) {
-    BtContextHandle btContextInReg = BtRegistry::getBtContext(btContext->getInfoHashAsString());
+    SharedHandle<BtRegistry> btRegistry = e->getBtRegistry();
+    BtContextHandle btContextInReg =
+      btRegistry->getBtContext(btContext->getInfoHashAsString());
     if(!btContextInReg.isNull() &&
        btContextInReg->getOwnerRequestGroup()->getGID() ==
 	btContext->getOwnerRequestGroup()->getGID()) {
-      BtRegistry::unregister(btContext->getInfoHashAsString());
+      btRegistry->unregister(btContext->getInfoHashAsString());
       if(!DHTRegistry::_peerAnnounceStorage.isNull()) {
-	DHTRegistry::_peerAnnounceStorage->removePeerAnnounce(btContext);
+	DHTRegistry::_peerAnnounceStorage->
+	  removeLocalPeerAnnounce(btContext->getInfoHash());
       }
     }
   }
@@ -878,15 +880,32 @@ DownloadResultHandle RequestGroup::createDownloadResult() const
 {
   std::deque<std::string> uris;
   getURIs(uris);
+
+  uint64_t sessionDownloadLength = 0;
+
+#ifdef ENABLE_BITTORRENT
+  if(!_peerStorage.isNull()) {
+    sessionDownloadLength =
+      _peerStorage->calculateStat().getSessionDownloadLength();
+  } else
+#endif // ENABLE_BITTORRENT
+    if(!_segmentMan.isNull()) {
+      sessionDownloadLength =
+	_segmentMan->calculateSessionDownloadLength();
+    }
+
   return
-    SharedHandle<DownloadResult>(new DownloadResult(_gid,
-						    getFilePath(),
-						    getTotalLength(),
-						    uris.empty() ? A2STR::NIL:uris.front(),
-						    uris.size(),
-						    downloadFinished()?
-						    DownloadResult::FINISHED :
-						    DownloadResult::NOT_YET));
+    SharedHandle<DownloadResult>
+    (new DownloadResult(_gid,
+			getFilePath(),
+			getTotalLength(),
+			uris.empty() ? A2STR::NIL:uris.front(),
+			uris.size(),
+			sessionDownloadLength,
+			_downloadContext->calculateSessionTime(),
+			downloadFinished()?
+			DownloadResult::FINISHED :
+			DownloadResult::NOT_YET));
 }
 
 void RequestGroup::registerServerHost(const ServerHostHandle& serverHost)
@@ -1058,4 +1077,19 @@ void RequestGroup::increaseAndValidateFileNotFoundCount()
   }
 }
 
+unsigned int RequestGroup::getNumConcurrentCommand() const
+{
+  return _numConcurrentCommand;
+}
+
+void RequestGroup::markInMemoryDownload()
+{
+  _inMemoryDownload = true;
+}
+
+bool RequestGroup::inMemoryDownload() const
+{
+  return _inMemoryDownload;
+}
+
 } // namespace aria2
