@@ -36,11 +36,13 @@
 #define _D_REQUEST_GROUP_H_
 
 #include "common.h"
+
+#include <string>
+#include <deque>
+
 #include "SharedHandle.h"
 #include "TransferStat.h"
 #include "TimeA2.h"
-#include <string>
-#include <deque>
 
 namespace aria2 {
 
@@ -62,6 +64,10 @@ class CheckIntegrityEntry;
 class DownloadResult;
 class ServerHost;
 class URISelector;
+#ifdef ENABLE_BITTORRENT
+class BtRuntime;
+class PeerStorage;
+#endif // ENABLE_BITTORRENT
 
 class RequestGroup {
 private:
@@ -120,6 +126,16 @@ private:
 
   unsigned int _fileNotFoundCount;
 
+#ifdef ENABLE_BITTORRENT
+  WeakHandle<BtRuntime> _btRuntime;
+
+  WeakHandle<PeerStorage> _peerStorage;
+#endif // ENABLE_BITTORRENT
+
+  // This flag just indicates that the downloaded file is not saved disk but
+  // just sits in memory.
+  bool _inMemoryDownload;
+
   const Option* _option;
 
   Logger* _logger;
@@ -203,6 +219,8 @@ public:
     _numConcurrentCommand = num;
   }
 
+  unsigned int getNumConcurrentCommand() const;
+
   int32_t getGID() const
   {
     return _gid;
@@ -287,7 +305,7 @@ public:
 
   bool isDependencyResolved();
 
-  void releaseRuntimeResource();
+  void releaseRuntimeResource(DownloadEngine* e);
 
   void postDownloadProcessing(std::deque<SharedHandle<RequestGroup> >& groups);
 
@@ -372,6 +390,12 @@ public:
   void updateLastModifiedTime(const Time& time);
 
   void increaseAndValidateFileNotFoundCount();
+
+  // Just set inMemoryDownload flag true.
+  void markInMemoryDownload();
+
+  // Returns inMemoryDownload flag.
+  bool inMemoryDownload() const;
 };
 
 typedef SharedHandle<RequestGroup> RequestGroupHandle;
