@@ -33,6 +33,14 @@
  */
 /* copyright --> */
 #include "RequestGroupMan.h"
+
+#include <iomanip>
+#include <sstream>
+#include <ostream>
+#include <fstream>
+#include <numeric>
+#include <algorithm>
+
 #include "BtProgressInfoFile.h"
 #include "RecoverableException.h"
 #include "RequestGroup.h"
@@ -52,12 +60,7 @@
 #include "Option.h"
 #include "prefs.h"
 #include "File.h"
-#include <iomanip>
-#include <sstream>
-#include <ostream>
-#include <fstream>
-#include <numeric>
-#include <algorithm>
+#include "Util.h"
 
 namespace aria2 {
 
@@ -122,6 +125,7 @@ RequestGroupMan::getRequestGroups() const
 
 class ProcessStoppedRequestGroup {
 private:
+  DownloadEngine* _e;
   std::deque<SharedHandle<RequestGroup> >& _reservedGroups;
   std::deque<SharedHandle<DownloadResult> >& _downloadResults;
   Logger* _logger;
@@ -143,8 +147,10 @@ private:
   }
 public:
   ProcessStoppedRequestGroup
-  (std::deque<SharedHandle<RequestGroup> >& reservedGroups,
+  (DownloadEngine* e,
+   std::deque<SharedHandle<RequestGroup> >& reservedGroups,
    std::deque<SharedHandle<DownloadResult> >& downloadResults):
+    _e(e),
     _reservedGroups(reservedGroups),
     _downloadResults(downloadResults),
     _logger(LogFactory::getInstance()) {}
@@ -178,7 +184,7 @@ public:
       } catch(RecoverableException& ex) {
 	_logger->error(EX_EXCEPTION_CAUGHT, ex);
       }
-      group->releaseRuntimeResource();
+      group->releaseRuntimeResource(_e);
       _downloadResults.push_back(group->createDownloadResult());
     }
   }
@@ -227,14 +233,15 @@ void RequestGroupMan::updateServerStat()
 		CollectServerStat(this));
 }
 
-void RequestGroupMan::removeStoppedGroup()
+void RequestGroupMan::removeStoppedGroup(DownloadEngine* e)
 {
   size_t numPrev = _requestGroups.size();
 
   updateServerStat();
 
   std::for_each(_requestGroups.begin(), _requestGroups.end(),
-		ProcessStoppedRequestGroup(_reservedGroups, _downloadResults));
+		ProcessStoppedRequestGroup(e, _reservedGroups,
+					   _downloadResults));
 
   _requestGroups.erase(std::remove_if(_requestGroups.begin(),
 				      _requestGroups.end(),
@@ -264,7 +271,7 @@ void RequestGroupMan::configureRequestGroup
 void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
 {
   RequestGroups temp;
-  removeStoppedGroup();
+  removeStoppedGroup(e);
   unsigned int count = 0;
   for(int num = _maxSimultaneousDownloads-_requestGroups.size();
       num > 0 && _reservedGroups.size() > 0; --num) {
@@ -380,29 +387,75 @@ void RequestGroupMan::showDownloadResults(std::ostream& o) const
   // ===+====+=======================================================================
   o << "\n"
     <<_("Download Results:") << "\n"
-    << "gid|stat|path/URI" << "\n"
-    << "===+====+======================================================================" << "\n";
+    << "gid|stat|avg speed  |path/URI" << "\n"
+    << "===+====+===========+==========================================================" << "\n";
+
+  int ok = 0;
+  int err = 0;
+  int inpr = 0;
+
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr = _downloadResults.begin();
       itr != _downloadResults.end(); ++itr) {
-    o << formatDownloadResult((*itr)->result == DownloadResult::FINISHED ?
-			      MARK_OK : MARK_ERR, *itr) << "\n";
+    std::string status;
+    if((*itr)->result == DownloadResult::FINISHED) {
+      status = MARK_OK;
+      ++ok;
+    } else {
+      status = MARK_ERR;
+      ++err;
+    }
+    o << formatDownloadResult(status, *itr) << "\n";
   }
   for(RequestGroups::const_iterator itr = _requestGroups.begin();
       itr != _requestGroups.end(); ++itr) {
     DownloadResultHandle result = (*itr)->createDownloadResult();
-    o << formatDownloadResult(result->result == DownloadResult::FINISHED ?
-			      MARK_OK : MARK_INPR, result) << "\n";
+    std::string status;
+    if(result->result == DownloadResult::FINISHED) {
+      status = MARK_OK;
+      ++ok;
+    } else {
+      // Since this RequestGroup is not processed by ProcessStoppedRequestGroup,
+      // its download stop time is not reseted.
+      // Reset download stop time and assign sessionTime here.
+      (*itr)->getDownloadContext()->resetDownloadStopTime();
+      result->sessionTime =
+	(*itr)->getDownloadContext()->calculateSessionTime();
+      status = MARK_INPR;
+      ++inpr;
+    }
+    o << formatDownloadResult(status, result) << "\n";
+  }
+  if(ok > 0 || err > 0 || inpr > 0) {
+    o << "\n"
+      << _("Status Legend:") << "\n";
+
+    if(ok > 0) {
+      o << " (OK):download completed.";
+    }
+    if(err > 0) {
+      o << "(ERR):error occurred.";
+    }
+    if(inpr > 0) {
+      o << "(INPR):download in-progress.";
+    }
+    o << "\n";
   }
-  o << "\n"
-    << _("Status Legend:") << "\n"
-    << " (OK):download completed.(ERR):error occurred.(INPR):download in-progress." << "\n";
 }
 
 std::string RequestGroupMan::formatDownloadResult(const std::string& status, const DownloadResultHandle& downloadResult) const
 {
   std::stringstream o;
   o << std::setw(3) << downloadResult->gid << "|"
-    << std::setw(4) << status << "|";
+    << std::setw(4) << status << "|"
+    << std::setw(11);
+  if(downloadResult->sessionTime > 0) {
+    o << Util::abbrevSize
+      (downloadResult->sessionDownloadLength*1000/downloadResult->sessionTime)+
+      "B/s";
+  } else {
+    o << "n/a";
+  }
+  o << "|";
   if(downloadResult->result == DownloadResult::FINISHED) {
     o << downloadResult->filePath;
   } else {
