@@ -54,12 +54,17 @@
 # include <gnutls/gnutls.h>
 #endif // HAVE_LIBGNUTLS
 
+#include "SharedHandle.h"
 #include "a2io.h"
 #include "a2netcompat.h"
 #include "a2time.h"
 
 namespace aria2 {
 
+#ifdef ENABLE_SSL
+class TLSContext;
+#endif // ENABLE_SSL
+
 class SocketCore {
   friend bool operator==(const SocketCore& s1, const SocketCore& s2);
   friend bool operator!=(const SocketCore& s1, const SocketCore& s2);
@@ -85,16 +90,18 @@ private:
   bool _wantRead;
   bool _wantWrite;
 
+#if ENABLE_SSL
+  static SharedHandle<TLSContext> _tlsContext;
+#endif // ENABLE_SSL
+
 #ifdef HAVE_LIBSSL
   // for SSL
-  SSL_CTX* sslCtx;
   SSL* ssl;
 
   int sslHandleEAGAIN(int ret);
 #endif // HAVE_LIBSSL
 #ifdef HAVE_LIBGNUTLS
   gnutls_session_t sslSession;
-  gnutls_certificate_credentials_t sslXcred;
   char* peekBuf;
   size_t peekBufLength;
   size_t peekBufMax;
@@ -286,8 +293,10 @@ public:
    * Makes this socket secure.
    * If the system has not OpenSSL, then this method do nothing.
    * connection must be established  before calling this method.
+   *
+   * If you are going to verify peer's certificate, hostname must be supplied.
    */
-  bool initiateSecureConnection();
+  bool initiateSecureConnection(const std::string& hostname="");
 
   void prepareSecureConnection();
 
@@ -317,6 +326,10 @@ public:
    * readData() or writeData() and the socket needs to write more data.
    */
   bool wantWrite() const;
+
+#ifdef ENABLE_SSL
+  static void setTLSContext(const SharedHandle<TLSContext>& tlsContext);
+#endif // ENABLE_SSL
 };
 
 } // namespace aria2
