@@ -72,6 +72,23 @@ static std::string toBoolArg(const char* optarg)
   return arg;
 }
 
+static void overrideWithEnv(Option* op, const OptionParser& optionParser,
+			    const std::string& pref,
+			    const std::string& envName)
+{
+  char* value = getenv(envName.c_str());
+  if(value) {
+    try {
+      optionParser.findByName(pref)->parse(op, value);
+    } catch(Exception& e) {
+      std::cerr << "Caught Error while parsing environment variable"
+		<< " '" << envName << "'"
+		<< "\n"
+		<< e.stackTrace();
+    }
+  }
+}
+
 Option* option_processing(int argc, char* const argv[])
 {
   std::stringstream cmdstream;
@@ -107,23 +124,23 @@ Option* option_processing(int argc, char* const argv[])
       { PREF_HTTP_PROXY.c_str(), required_argument, &lopt, 1 },
       { PREF_HTTP_USER.c_str(), required_argument, &lopt, 2 },
       { PREF_HTTP_PASSWD.c_str(), required_argument, &lopt, 3 },
-      { PREF_HTTP_PROXY_USER.c_str(), required_argument, &lopt, 4 },
-      { PREF_HTTP_PROXY_PASSWD.c_str(), required_argument, &lopt, 5 },
+      { "http-proxy-user", required_argument, &lopt, 4 },
+      { "http-proxy-passwd", required_argument, &lopt, 5 },
       { PREF_HTTP_AUTH_SCHEME.c_str(), required_argument, &lopt, 6 },
       { PREF_REFERER.c_str(), required_argument, &lopt, 7 },
       { PREF_RETRY_WAIT.c_str(), required_argument, &lopt, 8 },
       { PREF_FTP_USER.c_str(), required_argument, &lopt, 9 },
       { PREF_FTP_PASSWD.c_str(), required_argument, &lopt, 10 },
       { PREF_FTP_TYPE.c_str(), required_argument, &lopt, 11 },
-      { PREF_FTP_PASV.c_str(), no_argument, NULL, 'p' },
-      { PREF_FTP_VIA_HTTP_PROXY.c_str(), required_argument, &lopt, 12 },
-      { PREF_HTTP_PROXY_METHOD.c_str(), required_argument, &lopt, 14 },
+      { PREF_FTP_PASV.c_str(), optional_argument, 0, 'p' },
+      { "ftp-via-http-proxy", required_argument, &lopt, 12 },
+      { "http-proxy-method", required_argument, &lopt, 14 },
       { PREF_LOWEST_SPEED_LIMIT.c_str(), required_argument, &lopt, 200 },
       { PREF_MAX_DOWNLOAD_LIMIT.c_str(), required_argument, &lopt, 201 },
       { PREF_FILE_ALLOCATION.c_str(), required_argument, 0, 'a' },
       { PREF_ALLOW_OVERWRITE.c_str(), required_argument, &lopt, 202 },
 #ifdef ENABLE_MESSAGE_DIGEST
-      { PREF_CHECK_INTEGRITY.c_str(), required_argument, &lopt, 203 },
+      { PREF_CHECK_INTEGRITY.c_str(), optional_argument, 0, 'V' },
       { PREF_REALTIME_CHUNK_CHECKSUM.c_str(), required_argument, &lopt, 204 },
 #endif // ENABLE_MESSAGE_DIGEST
       { PREF_CONTINUE.c_str(), no_argument, 0, 'c' },
@@ -161,6 +178,15 @@ Option* option_processing(int argc, char* const argv[])
       { PREF_CONNECT_TIMEOUT.c_str(), required_argument, &lopt, 224 },
       { PREF_MAX_FILE_NOT_FOUND.c_str(), required_argument, &lopt, 225 },
       { PREF_AUTO_SAVE_INTERVAL.c_str(), required_argument, &lopt, 226 },
+      { PREF_HTTPS_PROXY.c_str(), required_argument, &lopt, 227 },
+      { PREF_FTP_PROXY.c_str(), required_argument, &lopt, 228 },
+      { PREF_ALL_PROXY.c_str(), required_argument, &lopt, 229 },
+      { PREF_PROXY_METHOD.c_str(), required_argument, &lopt, 230 },
+      { PREF_CERTIFICATE.c_str(), required_argument, &lopt, 231 },
+      { PREF_PRIVATE_KEY.c_str(), required_argument, &lopt, 232 },
+      { PREF_CA_CERTIFICATE.c_str(), optional_argument, &lopt, 233 },
+      { PREF_CHECK_CERTIFICATE.c_str(), optional_argument, &lopt, 234 },
+      { PREF_NO_PROXY.c_str(), required_argument, &lopt, 235 },
 #if defined ENABLE_BITTORRENT || defined ENABLE_METALINK
       { PREF_SHOW_FILES.c_str(), no_argument, NULL, 'S' },
       { PREF_SELECT_FILE.c_str(), required_argument, &lopt, 21 },
@@ -174,7 +200,7 @@ Option* option_processing(int argc, char* const argv[])
       //{ "upload-limit".c_str(), required_argument, &lopt, 20 },
       { PREF_SEED_TIME.c_str(), required_argument, &lopt, 22 },
       { PREF_SEED_RATIO.c_str(), required_argument, &lopt, 23 },
-      { PREF_MAX_UPLOAD_LIMIT.c_str(), required_argument, &lopt, 24 },
+      { PREF_MAX_UPLOAD_LIMIT.c_str(), required_argument, 0, 'u' },
       { PREF_PEER_ID_PREFIX.c_str(), required_argument, &lopt, 25 },
       { PREF_ENABLE_PEER_EXCHANGE.c_str(), optional_argument, &lopt, 26 },
       { PREF_ENABLE_DHT.c_str(), optional_argument, &lopt, 27 },
@@ -203,7 +229,7 @@ Option* option_processing(int argc, char* const argv[])
       { 0, 0, 0, 0 }
     };
     c = getopt_long(argc, argv, 
-		    "Dd:o:l:s:pt:m:vh::ST:M:C:a:cU:ni:j:Z::P::q::R::",
+		    "Dd:o:l:s:pt:m:vh::ST:M:C:a:cU:ni:j:Z::P::q::R::V::u:",
 		    longOpts, &optIndex);
     if(c == -1) {
       break;
@@ -221,11 +247,15 @@ Option* option_processing(int argc, char* const argv[])
 	cmdstream << PREF_HTTP_PASSWD << "=" << optarg << "\n";
 	break;
       case 4:
-	cmdstream << PREF_HTTP_PROXY_USER << "=" << optarg << "\n";
-	break;
+	std::cout << "--http-proxy-user was deprecated. See --http-proxy,"
+		  << " --https-proxy, --ftp-proxy, --all-proxy options."
+		  << std::endl;
+	exit(EXIT_FAILURE);
       case 5: 
-	cmdstream << PREF_HTTP_PROXY_PASSWD << "=" << optarg << "\n";
-	break;
+	std::cout << "--http-proxy-passwd was deprecated. See --http-proxy,"
+		  << " --https-proxy, --ftp-proxy, --all-proxy options."
+		  << std::endl;
+	exit(EXIT_FAILURE);
       case 6:
 	cmdstream << PREF_HTTP_AUTH_SCHEME << "=" << optarg << "\n";
 	break;
@@ -245,11 +275,15 @@ Option* option_processing(int argc, char* const argv[])
 	cmdstream << PREF_FTP_TYPE << "=" << optarg << "\n";
 	break;
       case 12:
-	cmdstream << PREF_FTP_VIA_HTTP_PROXY << "=" << optarg << "\n";
-	break;
+	std::cout << "--ftp-via-http-proxy was deprecated."
+		  << " Use --http-proxy-method option instead."
+		  << std::endl;
+	exit(EXIT_FAILURE);
       case 14:
-	cmdstream << PREF_HTTP_PROXY_METHOD << "=" << optarg << "\n";
-	break;
+	std::cout << "--http-proxy-method was deprecated."
+		  << " Use --proxy-method option instead."
+		  << std::endl;
+	exit(EXIT_FAILURE);
       case 15:
 	cmdstream << PREF_LISTEN_PORT << "=" << optarg << "\n";
 	break;
@@ -268,9 +302,6 @@ Option* option_processing(int argc, char* const argv[])
       case 23:
 	cmdstream << PREF_SEED_RATIO << "=" << optarg << "\n";
 	break;
-      case 24:
-	cmdstream << PREF_MAX_UPLOAD_LIMIT << "=" << optarg << "\n";
-	break;
       case 25:
 	cmdstream << PREF_PEER_ID_PREFIX << "=" << optarg << "\n";
 	break;
@@ -335,9 +366,6 @@ Option* option_processing(int argc, char* const argv[])
       case 202:
 	cmdstream << PREF_ALLOW_OVERWRITE << "=" << optarg << "\n";
 	break;
-      case 203:
-	cmdstream << PREF_CHECK_INTEGRITY << "=" << optarg << "\n";
-	break;
       case 204:
 	cmdstream << PREF_REALTIME_CHUNK_CHECKSUM << "=" << optarg << "\n";
 	break;
@@ -409,6 +437,33 @@ Option* option_processing(int argc, char* const argv[])
       case 226:
 	cmdstream << PREF_AUTO_SAVE_INTERVAL << "=" << optarg << "\n";
 	break;
+      case 227:
+	cmdstream << PREF_HTTPS_PROXY << "=" << optarg << "\n";
+	break;
+      case 228:
+	cmdstream << PREF_FTP_PROXY << "=" << optarg << "\n";
+	break;
+      case 229:
+	cmdstream << PREF_ALL_PROXY << "=" << optarg << "\n";
+	break;
+      case 230:
+	cmdstream << PREF_PROXY_METHOD << "=" << optarg << "\n";
+	break;
+      case 231:
+	cmdstream << PREF_CERTIFICATE << "=" << optarg << "\n";
+	break;
+      case 232:
+	cmdstream << PREF_PRIVATE_KEY << "=" << optarg << "\n";
+	break;
+      case 233:
+	cmdstream << PREF_CA_CERTIFICATE << "=" << optarg << "\n";
+	break;
+      case 234:
+	cmdstream << PREF_CHECK_CERTIFICATE << "=" << toBoolArg(optarg) << "\n";
+	break;
+      case 235:
+	cmdstream << PREF_NO_PROXY << "=" << optarg << "\n";
+	break;
       }
       break;
     }
@@ -436,7 +491,7 @@ Option* option_processing(int argc, char* const argv[])
       cmdstream << PREF_MAX_TRIES << "=" << optarg << "\n";
       break;
     case 'p':
-      cmdstream << PREF_FTP_PASV << "=" << V_TRUE << "\n";
+      cmdstream << PREF_FTP_PASV << "=" << toBoolArg(optarg) << "\n";
       break;
     case 'S':
       cmdstream << PREF_SHOW_FILES << "=" << V_TRUE << "\n";
@@ -480,6 +535,12 @@ Option* option_processing(int argc, char* const argv[])
     case 'R':
       cmdstream << PREF_REMOTE_TIME << "=" << toBoolArg(optarg) << "\n";
       break;
+    case 'V':
+      cmdstream << PREF_CHECK_INTEGRITY << "=" << toBoolArg(optarg) << "\n";
+      break;
+    case 'u':
+      cmdstream << PREF_MAX_UPLOAD_LIMIT << "=" << optarg << "\n";
+      break;
     case 'v':
       showVersion();
       exit(EXIT_SUCCESS);
@@ -501,7 +562,6 @@ Option* option_processing(int argc, char* const argv[])
   }
 
   {
-    oparser.setOptionHandlers(OptionHandlerFactory::createOptionHandlers());
     if(!noConf) {
       std::string cfname = 
 	ucfname.empty() ?
@@ -532,6 +592,13 @@ Option* option_processing(int argc, char* const argv[])
 	exit(EXIT_FAILURE);
       }
     }
+    // Override configuration with environment variables.
+    overrideWithEnv(op, oparser, PREF_HTTP_PROXY, "http_proxy");
+    overrideWithEnv(op, oparser, PREF_HTTPS_PROXY, "https_proxy");
+    overrideWithEnv(op, oparser, PREF_FTP_PROXY, "ftp_proxy");
+    overrideWithEnv(op, oparser, PREF_ALL_PROXY, "all_proxy");
+    overrideWithEnv(op, oparser, PREF_NO_PROXY, "no_proxy");
+
     try {
       oparser.parse(op, cmdstream);
     } catch(OptionHandlerException& e) {
@@ -546,9 +613,6 @@ Option* option_processing(int argc, char* const argv[])
       exit(EXIT_FAILURE);
     }
   }
-  if(op->defined(PREF_HTTP_PROXY_USER)) {
-    op->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
-  }
   if(
 #ifdef ENABLE_BITTORRENT
      !op->defined(PREF_TORRENT_FILE) &&
