@@ -48,7 +48,9 @@ _(" -s, --split=N                Download a file using N connections. If more\n"
   "                              remaining URLs are used for backup. If less than\n"\
   "                              N URLs are given, those URLs are used more than\n"\
   "                              once so that N connections total are made\n"\
-  "                              simultaneously. Please see -j option too.")
+  "                              simultaneously. Please see -j option too.\n"\
+  "                              Please note that in Metalink download, this\n"\
+  "                              option has no effect and use -C option instead.")
 #define TEXT_RETRY_WAIT \
 _(" --retry-wait=SEC             Set the seconds to wait to retry after an error\n"\
   "                              has occured.")
@@ -57,17 +59,30 @@ _(" -t, --timeout=SEC            Set timeout in seconds.")
 #define TEXT_MAX_TRIES \
 _(" -m, --max-tries=N            Set number of tries. 0 means unlimited.")
 #define TEXT_HTTP_PROXY \
-_(" --http-proxy=HOST:PORT       Use HTTP proxy server. This affects all URLs.")
+_(" --http-proxy=PROXY           Use this proxy server for HTTP.\n"\
+  "                              See also  --all-proxy option.\n"\
+  "                              This affects all URLs.")
+#define TEXT_HTTPS_PROXY \
+_(" --https-proxy=PROXY          Use this proxy server for HTTPS.\n"\
+  "                              See also  --all-proxy option.\n"\
+  "                              This affects all URLs.")
+#define TEXT_FTP_PROXY \
+_(" --ftp-proxy=PROXY            Use this proxy server for FTP.\n"\
+  "                              See also  --all-proxy option.\n"\
+  "                              This affects all URLs.")
+#define TEXT_ALL_PROXY \
+_(" --all-proxy=PROXY            Use this proxy server in the all protocols.\n"\
+  "                              You can override this setting and specify a\n"\
+  "                              proxy server for particular proctol using\n"\
+  "                              --http-proxy, --https-proxy and --ftp-proxy\n"\
+  "                              options.\n"\
+  "                              This affects all URLs.")
 #define TEXT_HTTP_USER \
 _(" --http-user=USER             Set HTTP user. This affects all URLs.")
 #define TEXT_HTTP_PASSWD \
 _(" --http-passwd=PASSWD         Set HTTP password. This affects all URLs.")
-#define TEXT_HTTP_PROXY_USER \
-_(" --http-proxy-user=USER       Set HTTP proxy user. This affects all URLs.")
-#define TEXT_HTTP_PROXY_PASSWD \
-_(" --http-proxy-passwd=PASSWD   Set HTTP proxy password. This affects all URLs.")
-#define TEXT_HTTP_PROXY_METHOD \
-_(" --http-proxy-method=METHOD   Set the method to use in proxy request.")
+#define TEXT_PROXY_METHOD \
+_(" --proxy-method=METHOD        Set the method to use in proxy request.")
 #define TEXT_HTTP_AUTH_SCHEME \
 _(" --http-auth-scheme=SCHEME    Set HTTP authentication scheme. Currently, basic\n"\
   "                              is the only supported scheme.")
@@ -80,9 +95,8 @@ _(" --ftp-passwd=PASSWD          Set FTP password. This affects all URLs.")
 #define TEXT_FTP_TYPE \
 _(" --ftp-type=TYPE              Set FTP transfer type.")
 #define TEXT_FTP_PASV \
-_(" -p, --ftp-pasv               Use passive mode in FTP.")
-#define TEXT_FTP_VIA_HTTP_PROXY \
-_(" --ftp-via-http-proxy=METHOD  Use HTTP proxy in FTP.")
+_(" -p, --ftp-pasv[=true|false]  Use the passive mode in FTP. If false is given,\n"\
+  "                              the active mode will be used.")
 #define TEXT_LOWEST_SPEED_LIMIT \
 _(" --lowest-speed-limit=SPEED   Close connection if download speed is lower than\n"\
   "                              or equal to this value(bytes per sec).\n"\
@@ -142,8 +156,8 @@ _(" --enable-http-keep-alive[=true|false] Enable HTTP/1.1 persistent connection.
 #define TEXT_ENABLE_HTTP_PIPELINING \
 _(" --enable-http-pipelining[=true|false] Enable HTTP/1.1 pipelining.")
 #define TEXT_CHECK_INTEGRITY \
-_(" --check-integrity=true|false  Check file integrity by validating piece hash.\n"\
-  "                              This option only affects in BitTorrent downloads\n"\
+_(" -V, --check-integrity[=true|false] Check file integrity by validating piece\n"\
+  "                              hashes. This option has effect only in BitTorrent\n"\
   "                              and Metalink downloads with chunk checksums.\n"\
   "                              Use this option to re-download a damaged portion\n"\
   "                              of a file.")
@@ -166,10 +180,15 @@ _(" -n, --no-netrc               Disables netrc support.")
 _(" -i, --input-file=FILE        Downloads URIs found in FILE. You can specify\n"\
   "                              multiple URIs for a single entity: separate\n"\
   "                              URIs on a single line using the TAB character.\n"\
-  "                              Reads input from stdin when '-' is specified.")
+  "                              Reads input from stdin when '-' is specified.\n"\
+  "                              The additional out and dir options can be\n"\
+  "                              specified after each line of URIs. This optional\n"\
+  "                              line must start with white space(s). See INPUT\n"\
+  "                              FILE section of man page for details.")
 #define TEXT_MAX_CONCURRENT_DOWNLOADS \
 _(" -j, --max-concurrent-downloads=N Set maximum number of parallel downloads for\n"\
-  "                              every static (HTTP/FTP) URL, torrent and metalink.")
+  "                              every static (HTTP/FTP) URL, torrent and metalink.\n"\
+  "                              See also -s and -C options.")
 #define TEXT_LOAD_COOKIES \
 _(" --load-cookies=FILE          Load Cookies from FILE using the Firefox3 format\n"\
   "                              and Mozilla/Firefox(1.x/2.x)/Netscape format.")
@@ -208,7 +227,7 @@ _(" --listen-port=PORT...        Set TCP port number for BitTorrent downloads.\n
   "                              to specify a range: \"6881-6999\". ',' and '-' can\n"\
   "                              be used together.")
 #define TEXT_MAX_UPLOAD_LIMIT \
-_(" --max-upload-limit=SPEED     Set max upload speed in bytes per sec.\n"\
+_(" -u, --max-upload-limit=SPEED Set max upload speed in bytes per sec.\n"\
   "                              0 means unrestricted.\n"\
   "                              You can append K or M(1K = 1024, 1M = 1024K).")
 #define TEXT_SEED_TIME \
@@ -269,7 +288,13 @@ _(" --bt-seed-unverified[=true|false] Seed previously downloaded files without\n
 _(" -M, --metalink-file=METALINK_FILE The file path to the .metalink file.")
 #define TEXT_METALINK_SERVERS \
 _(" -C, --metalink-servers=NUM_SERVERS The number of servers to connect to\n"\
-  "                              simultaneously.")
+  "                              simultaneously. Some Metalinks regulates the\n"\
+  "                              number of servers to connect. aria2 strictly\n"\
+  "                              respects them. This means that if Metalink defines\n"\
+  "                              the maxconnections attribute lower than\n"\
+  "                              NUM_SERVERS, then aria2 uses the value of\n"\
+  "                              maxconnections attribute instead of NUM_SERVERS.\n"\
+  "                              See also -s and -j options.")
 #define TEXT_METALINK_VERSION \
 _(" --metalink-version=VERSION   The version of the file to download.")
 #define TEXT_METALINK_LANGUAGE \
@@ -378,3 +403,24 @@ _(" --auto-save-interval=SEC     Save a control file(*.aria2) every SEC seconds.
   "                              If 0 is given, a control file is not saved during\n"\
   "                              download. aria2 saves a control file when it stops\n"\
   "                              regardless of the value.")
+#define TEXT_CERTIFICATE \
+_(" --certificate=FILE           Use the client certificate in FILE.\n"\
+  "                              The certificate must be in PEM format.\n"\
+  "                              You may use --private-key option to specify the\n"\
+  "                              private key.")
+#define TEXT_PRIVATE_KEY \
+_(" --private-key=FILE           Use the private key in FILE.\n"\
+  "                              The private key must be decrypted and in PEM\n"\
+  "                              format. See also --certificate option.")
+#define TEXT_CA_CERTIFICATE \
+_(" --ca-certificate=FILE        Use the certificate authorities in FILE to verify\n"\
+  "                              the peers. The certificate file must be in PEM\n"\
+  "                              format and can contain multiple CA certificates.\n"\
+  "                              Use --check-certificate option to enable\n"\
+  "                              verification.")
+#define TEXT_CHECK_CERTIFICATE \
+_(" --check-certificate[=true|false] Verify the peer using certificates specified\n"\
+  "                              in --ca-certificate option.")
+#define TEXT_NO_PROXY \
+_(" --no-proxy=DOMAINS           Specify comma separated hostnames or domains to\n"\
+  "                              which proxy should not be used.")
