@@ -1,4 +1,7 @@
 #include "HttpRequest.h"
+
+#include <cppunit/extensions/HelperMacros.h>
+
 #include "prefs.h"
 #include "AuthConfigFactory.h"
 #include "PiecedSegment.h"
@@ -8,7 +11,6 @@
 #include "Option.h"
 #include "array_fun.h"
 #include "CookieStorage.h"
-#include <cppunit/extensions/HelperMacros.h>
 
 namespace aria2 {
 
@@ -30,15 +32,12 @@ class HttpRequestTest : public CppUnit::TestFixture {
   CPPUNIT_TEST_SUITE_END();
 private:
   SharedHandle<Option> _option;
+  SharedHandle<AuthConfigFactory> _authConfigFactory;
 public:
   void setUp()
   {
-    _option.reset(new Option());
-    
-    SharedHandle<AuthConfigFactory> authConfigFactory
-      (new AuthConfigFactory(_option.get()));
-    SingletonHolder<SharedHandle<AuthConfigFactory> >::instance
-      (authConfigFactory);
+    _option.reset(new Option());    
+    _authConfigFactory.reset(new AuthConfigFactory(_option.get()));
   }
 
   void testGetStartByte();
@@ -107,12 +106,6 @@ void HttpRequestTest::testCreateRequest()
 {
   SharedHandle<Piece> p;
 
-  _option->put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
-  _option->put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
-  _option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
-  _option->put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
-  _option->put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
-
   SharedHandle<Request> request(new Request());
   request->supportsPersistentConnection(true);
 
@@ -125,6 +118,7 @@ void HttpRequestTest::testCreateRequest()
   httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
 
   // remove "Connection: close" and add end byte range
   request->setPipeliningHint(true);  
@@ -225,8 +219,6 @@ void HttpRequestTest::testCreateRequest()
   _option->put(PREF_HTTP_USER, "aria2user");
   _option->put(PREF_HTTP_PASSWD, "aria2passwd");
 
-  httpRequest.configure(_option.get());
-
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
@@ -240,41 +232,10 @@ void HttpRequestTest::testCreateRequest()
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
   // enable http proxy auth
-  _option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
-
-  httpRequest.configure(_option.get());
-
-  expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
-    "User-Agent: aria2\r\n"
-    "Accept: */*\r\n"
-    "Host: localhost:8080\r\n"
-    "Pragma: no-cache\r\n"
-    "Cache-Control: no-cache\r\n"
-    "Connection: close\r\n"
-    "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
-    "\r\n";
-
-  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
-
-  _option->put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
-
-  httpRequest.configure(_option.get());
-
-  expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
-    "User-Agent: aria2\r\n"
-    "Accept: */*\r\n"
-    "Host: localhost:8080\r\n"
-    "Pragma: no-cache\r\n"
-    "Cache-Control: no-cache\r\n"
-    "Connection: close\r\n"
-    "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
-    "\r\n";
-
-  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
-
-  _option->put(PREF_HTTP_PROXY_METHOD, V_GET);
-
-  httpRequest.configure(_option.get());
+  SharedHandle<Request> proxyRequest(new Request());
+  CPPUNIT_ASSERT(proxyRequest->setUrl
+		 ("http://aria2proxyuser:aria2proxypasswd@localhost:9000"));
+  httpRequest.setProxyRequest(proxyRequest);
 
   expectedText = "GET http://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -308,9 +269,8 @@ void HttpRequestTest::testCreateRequest()
 
   request->setPipeliningHint(false);
 
-  _option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
-
-  httpRequest.configure(_option.get());
+  // turn off proxy auth
+  CPPUNIT_ASSERT(proxyRequest->setUrl("http://localhost:9000"));
 
   expectedText = "GET http://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -328,17 +288,16 @@ void HttpRequestTest::testCreateRequest()
 
 void HttpRequestTest::testCreateRequest_ftp()
 {
-  _option->put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
-  _option->put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
-  _option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
-  _option->put(PREF_FTP_USER, "aria2user");
-  _option->put(PREF_FTP_PASSWD, "aria2passwd");
-  _option->put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
-  _option->put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
+   _option->put(PREF_FTP_USER, "aria2user");
+   _option->put(PREF_FTP_PASSWD, "aria2passwd");
 
   SharedHandle<Request> request(new Request());
   request->setUrl("ftp://localhost:8080/archives/aria2-1.0.0.tar.bz2");
 
+  SharedHandle<Request> proxyRequest(new Request());
+  CPPUNIT_ASSERT(proxyRequest->setUrl
+		 ("http://localhost:9000"));
+
   HttpRequest httpRequest;
   SharedHandle<Piece> p(new Piece(0, 1024*1024));
   SharedHandle<Segment> segment
@@ -346,8 +305,8 @@ void HttpRequestTest::testCreateRequest_ftp()
   httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
-
-  httpRequest.configure(_option.get());
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
+  httpRequest.setProxyRequest(proxyRequest);
 
   std::string expectedText =
     "GET ftp://aria2user@localhost:8080/archives/aria2-1.0.0.tar.bz2"
@@ -358,17 +317,15 @@ void HttpRequestTest::testCreateRequest_ftp()
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
     "Connection: close\r\n"
+    "Proxy-Connection: close\r\n"
     "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  // How to enable HTTP proxy authorization in FTP download via HTTP proxy
-  _option->put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
-  _option->put(PREF_HTTP_PROXY_METHOD, V_GET);
-  _option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
-
-  httpRequest.configure(_option.get());
+  // test proxy authorization
+  CPPUNIT_ASSERT(proxyRequest->setUrl
+		 ("http://aria2proxyuser:aria2proxypasswd@localhost:9000"));
 
   expectedText =
     "GET ftp://aria2user@localhost:8080/archives/aria2-1.0.0.tar.bz2"
@@ -385,7 +342,6 @@ void HttpRequestTest::testCreateRequest_ftp()
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
-  
 }
 
 void HttpRequestTest::testCreateRequest_with_cookie()
@@ -413,6 +369,7 @@ void HttpRequestTest::testCreateRequest_with_cookie()
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
   httpRequest.setCookieStorage(st);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
 
   std::string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -478,6 +435,7 @@ void HttpRequestTest::testCreateRequest_query()
   HttpRequest httpRequest;
   httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
 
   std::string expectedText =
     "GET /wiki?id=9ad5109a-b8a5-4edf-9373-56a1c34ae138 HTTP/1.1\r\n"
@@ -499,17 +457,21 @@ void HttpRequestTest::testCreateProxyRequest()
   SharedHandle<Piece> p(new Piece(0, 1024*1024));
   SharedHandle<Segment> segment(new PiecedSegment(1024*1024, p));
 
+  SharedHandle<Request> proxyRequest(new Request());
+  CPPUNIT_ASSERT(proxyRequest->setUrl("http://localhost:9000"));
+
   HttpRequest httpRequest;
 
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
+  httpRequest.setProxyRequest(proxyRequest);
 
   request->supportsPersistentConnection(true);
 
   std::string expectedText = "CONNECT localhost:80 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Host: localhost:80\r\n"
-    "Proxy-Connection: close\r\n"
+    //"Proxy-Connection: close\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createProxyRequest());
@@ -520,7 +482,7 @@ void HttpRequestTest::testCreateProxyRequest()
   expectedText = "CONNECT localhost:80 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Host: localhost:80\r\n"
-    "Proxy-Connection: Keep-Alive\r\n"
+    //"Proxy-Connection: Keep-Alive\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createProxyRequest());
@@ -532,10 +494,23 @@ void HttpRequestTest::testCreateProxyRequest()
   expectedText = "CONNECT localhost:80 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Host: localhost:80\r\n"
-    "Proxy-Connection: Keep-Alive\r\n"
+    //"Proxy-Connection: Keep-Alive\r\n"
     "\r\n";
 
-  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createProxyRequest());  
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createProxyRequest());
+
+  // test proxy authorization
+  CPPUNIT_ASSERT(proxyRequest->setUrl
+		 ("http://aria2proxyuser:aria2proxypasswd@localhost:9000"));
+
+  expectedText = "CONNECT localhost:80 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Host: localhost:80\r\n"
+    //"Proxy-Connection: Keep-Alive\r\n"
+    "Proxy-Authorization: Basic YXJpYTJwcm94eXVzZXI6YXJpYTJwcm94eXBhc3N3ZA==\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createProxyRequest());
 }
 
 void HttpRequestTest::testIsRangeSatisfied()
@@ -605,6 +580,7 @@ void HttpRequestTest::testUserAgent()
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
   httpRequest.setUserAgent("aria2 (Linux)");
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
 
   std::string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2 (Linux)\r\n"
@@ -617,10 +593,15 @@ void HttpRequestTest::testUserAgent()
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
+  SharedHandle<Request> proxyRequest(new Request());
+  CPPUNIT_ASSERT(proxyRequest->setUrl("http://localhost:9000"));
+  
+  httpRequest.setProxyRequest(proxyRequest);
+
   std::string expectedTextForProxy = "CONNECT localhost:8080 HTTP/1.1\r\n"
     "User-Agent: aria2 (Linux)\r\n"
     "Host: localhost:8080\r\n"
-    "Proxy-Connection: close\r\n"
+    //"Proxy-Connection: close\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedTextForProxy, httpRequest.createProxyRequest());
@@ -634,6 +615,7 @@ void HttpRequestTest::testAddHeader()
   HttpRequest httpRequest;
   httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
   httpRequest.addHeader("X-ARIA2: v0.13\nX-ARIA2-DISTRIBUTE: enabled\n");
 
   std::string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
@@ -661,7 +643,9 @@ void HttpRequestTest::testAddAcceptType()
   HttpRequest httpRequest;
   httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
-  httpRequest.addAcceptType(&acceptTypes[0], &acceptTypes[arrayLength(acceptTypes)]);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
+  httpRequest.addAcceptType(&acceptTypes[0],
+			    &acceptTypes[arrayLength(acceptTypes)]);
 
   std::string expectedText =
     "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
@@ -683,6 +667,7 @@ void HttpRequestTest::testEnableAcceptEncoding()
 
   HttpRequest httpRequest;
   httpRequest.setRequest(request);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
 
   std::string acceptEncodings;
 #ifdef HAVE_LIBZ
